@@ -74,7 +74,7 @@ enum {
  * it gets called. We'll only call it every now and then to not loose a
  * lot of performance
  */
-#define UPDATE_PERIOD ((gint64) (100 * 1000))
+#define UPDATE_PERIOD ((gint64) (500 * 1000))
 
 static gint64
 system_time (void)
@@ -180,7 +180,7 @@ call_progress_with_current_names (GnomeVFSProgressCallbackState *progress, Gnome
 
 	result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 
-	progress->next_update_callback_time = system_time () + progress->update_callback_period;
+	progress->next_text_update_callback_time = system_time () + progress->update_callback_period;
 	progress->next_update_callback_time = progress->next_text_update_callback_time;
 	
 	progress->progress_info->phase = phase;
@@ -1341,6 +1341,23 @@ xfer_open_source (GnomeVFSHandle **source_handle,
 		}
 	} while (retry);
 
+	/* if we didn't get a file size earlier,
+	 * try to get one from the handle (#330498)
+	 */
+	if (progress->progress_info->file_size == 0) {
+		GnomeVFSFileInfo *info;
+
+		info = gnome_vfs_file_info_new ();
+		result = gnome_vfs_get_file_info_from_handle (*source_handle,
+							      info,
+							      GNOME_VFS_FILE_INFO_DEFAULT);
+		if (result == GNOME_VFS_OK &&
+		    info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) {
+			progress->progress_info->file_size = info->size;
+		}
+		gnome_vfs_file_info_unref (info);
+	}
+
 	return result;
 }
 
@@ -1813,8 +1830,12 @@ copy_items (const GList *source_uri_list,
 					result = GNOME_VFS_ERROR_INTERRUPTED;
 				}
 
-				overwrite_mode_abort = GNOME_VFS_XFER_OVERWRITE_MODE_ABORT;
-				
+				/* If we're creating unique names, always abort on overwrite and
+				 * calculate a new name */
+				overwrite_mode_abort = overwrite_mode;
+				if ((xfer_options & GNOME_VFS_XFER_USE_UNIQUE_NAMES) != 0) {
+					overwrite_mode_abort = GNOME_VFS_XFER_OVERWRITE_MODE_ABORT;
+				} 
 				
 				if (info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
 					result = copy_file (info, target_dir_info,
@@ -1846,8 +1867,7 @@ copy_items (const GList *source_uri_list,
 					break;
 				}
 
-				if (overwrite_mode != GNOME_VFS_XFER_OVERWRITE_MODE_QUERY
-				    || (xfer_options & GNOME_VFS_XFER_USE_UNIQUE_NAMES) == 0) {
+				if ((xfer_options & GNOME_VFS_XFER_USE_UNIQUE_NAMES) == 0) {
 					gnome_vfs_uri_unref (target_uri);
 					break;
 				}
